unit DeKiBulleUtils;

interface

Uses  Windows, Classes, Graphics, Messages;

Procedure Associe(Extension : String;Executable : String; Icone : String);
Procedure RetireAssociation(Extension : String);
function  Split(S: string; d: Pchar): TStringList;
function  NomFichierSon(F: string): string;
function  ValideExt(const F: string): boolean;
procedure GestionLangue(Index : integer = 0);
procedure MiseaJoursCouleurs(Totale : Boolean = True);
procedure Executer(const action : string);
function  GetGenreFromIndex(Index : integer) : string;
function  GetIndexofGenre(const Genre : String) : byte;
procedure SetEqualizer;

const
  SGenre : array[0..80] of string[17] = ('Blues','Classic Rock','Country','Dance','Disco','Funk','Grunge','Hip-Hop','Jazz','Metal','New Age','Oldies','Other','Pop','R&B','Rap','Reggae','Rock','Techno','Industrial','Alternative','Ska','Death Metal','Pranks','Soundtrack','Euro-Techno','Ambient','Trip-Hop','Vocal','Jazz+Funk','Fusion','Trance','Classical','Instrumenta','Acid','House','Game','Sound Clip','Gospel','Noise','AlternRock','Bass','Soul','Punk','Space','Meditative','Instrumental Pop','Instrumental Rock','Ethnic','Gothic','Darkwave','Techno-Industrial','Electronic','Pop-Folk','Eurodance','Dream','Southern Rock','Comedy','Cult','Gangsta','Top 40','Christian Rap','Pop/Funk','Jungle','Native American','Cabaret','New Wave','Psychadelic','Rave','Showtunes','Trailer','Lo-Fi','Tribal','Acid Punk','Acid Jazz','Polka','Retro','Musical','Rock & Roll','Hard Rock','Unknow');

implementation

Uses  Registry, Dialogs, ID3v1, ID3v2, SysUtils, IniFiles, Forms, Controls, ShellAPI,
      Unit1, Unit2, Unit3, Unit4, InfoWMA, InfoOGG, Bass, StrUtils;

{-----------------------------------------------------------------}
{          Procdure permettant d'associer une extension          }
{           un executable.   Ex: *.pas -> DELPHI                 }
{-----------------------------------------------------------------}
Procedure Associe(Extension : String;Executable : String; Icone : String);
Var
   Registre : TRegistry;
Begin
   Registre := TRegistry.Create;
   with Registre do
   begin
      RootKey := HKEY_CLASSES_ROOT;
      OpenKey('.' + Extension, True);
        WriteString('', Extension + '_file');                 // associe les ".ext" a "ext_file"
      CloseKey;
      OpenKey(Extension + '_file\shell\open\command', True);
        WriteString('', '"' + Executable + '"' + ' "%1"');    // cre le type de fichier "ext_file"
      CloseKey;
      OpenKey(Extension + '_file\DefaultIcon', True);
        WriteString('', Icone);
      CloseKey;
   end;
   Registre.Free;
End;

{-----------------------------------------------------------------}
{          Procdure permettant de retirer une extension          }
{          d'un executable.                                       }
{-----------------------------------------------------------------}
Procedure RetireAssociation(Extension : String);
Var
  Registre:TRegistry;
Begin
  Registre := TRegistry.Create;          // Cration de Registre
  Try
    Registre.RootKey := HKEY_CLASSES_ROOT; //spcifie la clef racine choisi pour l'application
    Registre.DeleteKey('.' + extension);   //efface le noeud
    Registre.Free;
    Registre:=TRegistry.Create;
    Registre.RootKey := HKEY_CLASSES_ROOT;
    Registre.DeleteKey(extension + '_file\shell\open\command'); //efface le noeud
    Registre.DeleteKey(extension + '_file\shell\open');
    Registre.DeleteKey(extension + '_file\shell');
    Registre.DeleteKey(extension + '_file');
    Registre.CloseKey; // Ecrit la cl en cours dans le registre et ferme cette cl.
    Registre.Free;     // libration de la mmoire (annule l'effet de create)
  Except
    Showmessage('Erreur');
    Registre.Free;  // libre la mmoire mme en cas d'erreur.
  End;
End;

{------------------------------------------------------------------------}
{       fonction Split                                                   }
{------------------------------------------------------------------------}
function Split(S: string; d: Pchar): TStringList;
var
    Spliti: integer;
    Splitt: string;
begin
    Result := TStringList.Create;
    Splitt := '';
    for Spliti := 1 to Length(S) do   // une string a commence  1...
    begin
        if S[Spliti] = d then
        begin
            if Splitt <> '' then
            begin
                Result.Add(Splitt);
                Splitt := '';
            end;
        end else
            Splitt := Splitt + S[Spliti];
    end;
    if Splitt <> '' then
        Result.Add(Splitt);
end;

//============================================================================//
{             Renvoie le nom d'un fichier son  afficher                       }
//----------------------------------------------------------------------------//
function NomFichierSon(F: string): string;
var
    MP3Tag : TagMP3;
    MP3Tag2: TID3v2;
    WMATag : InfosWMA;
    OGGTag : InfosOGG;
    Ext    : string;
    n      : TstringList;
begin
        { Est-ce que le fichier existe ? }
    if not(FileExists(F)) then
    begin
        Result := 'No File...';
        Exit;
    end;

    Ext := LowerCase(ExtractFileExt(F));

        { Si le fichier est un cda (CD audio)  }
    if AnsiContainsText('$.cda$', Ext) then
    begin
        Result := Format('CD Audio - %s', [Copy(F, 1, Length(F) - Length(ExtractFileExt(F)))]);
        Exit;
    end;

        { Si le fichier est un mp3 ou mp2 on tente de regarder les TAG ID3 v2 }
    if AnsiContainsText('$.mp3$.mp2$', Ext) then
    begin
            { Lecture TAG ID3 v2 }
        MP3Tag2 := TID3v2.Create;
        With MP3Tag2 do begin
            ReadFromFile(F);
            if (Exists) and (Trim(Title) <> '') then
            begin
                if Track > 0 then
                    Result := Format('%s - %s - %.2d - %s', [Artist, Album, Track, Title])
                else
                    Result := Format('%s - %s - %s', [Artist, Album, Title]);
                Free;
                Exit;
            end;
            Free;
        end;
    end;

        { Si le fichier est un mp3 ou mp2 ou mpc on tente de regarder les TAG ID3 v1.1 }
    if AnsiContainsText('$.mp3$.mp2$.mpc$', Ext) then
    begin
            { Lecture TAG ID3 v1.1 }
        MP3Tag := Lire_Tag(F);
        With MP3Tag do begin
            if Trim(Titre) <> '' then
            begin
                if TrackNb > 0 then
                    Result := Format('%s - %s - %.2d - %s', [Trim(Artiste), Trim(Album), TrackNb, Trim(Titre)])
                else
                    Result := Format('%s - %s - %s', [Trim(Artiste), Trim(Album), Trim(Titre)]);
                Exit;
            end;
        end;
    end;

        { Si le fichier est un wma on tente de regarder les TAG }
    if AnsiContainsText('$.wma$', Ext) then
    begin
        WMATag := LireInfosWMA(F);
        With WMATag do begin
            if Trim(Title) <> '' then
            begin
                if Length(Track) = 1 then Track := '0' + Track;
                if Track <> '' then
                    Result := Format('%s - %s - %s - %s', [Artist, Album, Track, Title])
                else
                    Result := Format('%s - %s - %s', [Artist, Album, Title]);
                Exit;
            end;
        end;
    end;

        { Si le fichier est un ogg on tente de regarder les TAG }
    if AnsiContainsText('$.ogg$', Ext) then
    begin
        OGGTag := LireInfosOGG(F);
        With OGGTag do begin
            if Trim(Title) <> '' then
            begin
                if Length(Track) = 1 then Track := '0' + Track;
                if Track <> '' then
                    Result := Format('%s - %s - %s - %s', [Artist, Album, Track, Title])
                else
                    Result := Format('%s - %s - %s', [Artist, Album, Title]);
                Exit;
            end;
        end;
    end;

        { Pas de TAGs du tout }
        { Lecture du chemin du fichier }
        { On suppose que le chemin est : ARTISTE\ALBUM\CHANSON.mp3 }
    n := Split(F, '\');
    F := ExtractFileName(F);
    With n do
    begin
      if Count > 3 then
        Result := Format('%s - %s - %s', [Strings[Count - 3], Strings[Count - 2], Copy(F, 1, Length(F) - Length(ExtractFileExt(F)))])
      else
        Result := Copy(F, 1, Length(F) - Length(ExtractFileExt(F)));
      Free;
    end;

end;

//============================================================================//
{             Valide l'extension d'un fichier                                  }
//----------------------------------------------------------------------------//
function ValideExt(const F: string) : boolean;
var
    i : integer;
    E : string[5];
begin
    E := LowerCase(ExtractFileExt(F));
    for i := Low(VEXT) to High(VEXT) do
    begin
        if E = '.' + VEXT[i] then
        begin
            Result := true;
            Exit;
        end;
    end;
    Result := false;
end;

//============================================================================//
{                  Gestion de la langue                                        }
//----------------------------------------------------------------------------//
procedure GestionLangue(Index : integer = 0);
var
    IniLangue: TIniFile;
    i        : integer;
begin
            { Gestion de la Langue }
    IniLangue := TiniFile.Create(FichierLangue);
    with IniLangue do
    begin
        if (Index = 0) or (Index = 1) then
        begin
                    { Popup Menu }
            With Form1 do
            begin
            MenuAfficher.Caption       := ReadString('MENU', 'MenuAfficher', 'Afficher');
            MenuOptions.Caption        := ReadString('MENU', 'MenuOptions', 'Options');
            MenuPlayPause.Caption      := ReadString('MENU', 'MenuPlayPause', 'Lecture / Pause');
            MenuQuitter.Caption        := ReadString('MENU', 'MenuQuitter', 'Quitter');
            MenuShuffle.Caption        := ReadString('MENU', 'MenuShuffle', 'Alatoire');
            MenuAfficherBallon.Caption := ReadString('MENU', 'MenuAfficherBallon', 'Afficher ballon');
            MenuSousRep.Caption        := ReadString('MENU', 'MenuSousRep', 'Inclure sous-rpertoires');

                    { Popup Menu liste des rpertoires }
            MenuMettreSuite.Caption := ReadString('MENU', 'MenuMettreSuite', 'Mettre  la suite');
            MenuLireRep.Caption     := ReadString('MENU', 'MenuLireRep', 'Lire le contenu');
            MenuExplorerRep.Caption := ReadString('MENU', 'MenuExplorerRep', 'Explorer le rpertoire');
            MenuGererRep.Caption    := ReadString('MENU', 'MenuGererRep', 'Grer les rpertoires');
            MenuRefreshRep.Caption  := ReadString('MENU', 'MenuRefresh', 'Rafrachir');

                    { Popup Menu liste de lecture }
            MenuEcouterChanson.Caption   := ReadString('MENU', 'MenuEcouterChanson', 'Ecouter');
            MenuEditeurTag.Caption       := ReadString('MENU', 'MenuEditeurTag', 'Editeur de tag MP3/OGG');
            MenuSelectAll.Caption        := ReadString('MENU', 'MenuSelectAll', 'Slectionner tout');
            MenuSupprimerChanson.Caption := ReadString('MENU', 'MenuSupprimerChanson', 'Supprimer la slection');
            MenuOuvrirFichier.Caption    := ReadString('MENU', 'MenuOuvrirFichier', 'Ouvrir un fichier');
            MenuSauverListe.Caption      := ReadString('MENU', 'MenuSauverListe', 'Sauver la playliste');
            MenuRefreshListe.Caption     := ReadString('MENU', 'MenuRefresh', 'Rafrachir');

                    { Main Form }
            SpdBtnRep.Hint       := ReadString('MAIN', 'SpdBtnRep', 'Rpertoires');
            SpdBtnListe.Hint     := ReadString('MAIN', 'SpdBtnListe', 'Playliste');
            SpdBtnPrev.Hint      := ReadString('MAIN', 'SpdBtnPrev', 'Prcdente');
            SpdBtnPlay.Hint      := ReadString('MAIN', 'SpdBtnPlay', 'Lecture / Pause');
            SpdBtnNext.Hint      := ReadString('MAIN', 'SpdBtnNext', 'Suivante');
            ProgressBar1.Hint    := ReadString('MAIN', 'ProgressBar1', 'Position');
            ProgressBar2.Hint    := ReadString('MAIN', 'ProgressBar2', 'Volume');
            SpdBtnEgaliseur.Hint := ReadString('MAIN', 'SpdBtnEgaliseur', 'Egaliseur');
            SpdBtnOptions.Hint   := ReadString('MAIN', 'SpdBtnOptions', 'Options');
            end;
        end;
        if (Index = 0) or (Index = 2) then
        begin
                    { Nom Form Options }
            with Form2 do
            begin
            Caption := 'DKiBulle - ' + ReadString('OPTION', 'Page0', 'Options');

                    { Options Onglet 1 }
            TabbedNotebook1.Pages.Strings[0] := ReadString('OPTION', 'Page0', 'Options');

            RadioGroup1.Caption          := ReadString('OPTION', 'RadioGroup1', 'Dmarrer avec windows');
            RadioGroup1.Items.Strings[0] := ReadString('OPTION', 'RadioGroup1Item0', 'Oui');
            RadioGroup1.Items.Strings[1] := ReadString('OPTION', 'RadioGroup1Item1', 'Non');

            GroupBox6.Caption    := ReadString('OPTION', 'GroupBox6', 'Raccourcis clavier');
            CheckBox1.Caption    := ReadString('OPTION', 'CheckBox1', 'Lecture/Pause');
            CheckBox2.Caption    := ReadString('OPTION', 'CheckBox2', 'Prcdente');
            CheckBox3.Caption    := ReadString('OPTION', 'CheckBox3', 'Suivante');
            CheckBox4.Caption    := ReadString('OPTION', 'CheckBox4', 'Volume +');
            CheckBox5.Caption    := ReadString('OPTION', 'CheckBox5', 'Volume -');
            CheckBox6.Caption    := ReadString('OPTION', 'CheckBox6', 'Afficher/Cacher');

            GroupBox7.Caption    := ReadString('OPTION', 'GroupBox7', 'Autres');
            CheckBallon.Caption  := ReadString('OPTION', 'CheckBallon', 'Afficher ballon');
            CheckSousRep.Caption := ReadString('OPTION', 'CheckSousRep', 'Inclure sous-rpertoires');
            CheckShuffle.Caption := ReadString('OPTION', 'CheckShuffle', 'Lecture alatoire');

            GroupBox8.Caption := ReadString('OPTION', 'GroupBox8', 'Langues');

            GroupBox10.Caption            := ReadString('OPTION', 'GroupBox10', 'Associations avec les fichiers audio');
            BtnAssocier.Caption           := ReadString('OPTION', 'BtnAssocier', 'Associer');
            BtnRetirerAssociation.Caption := ReadString('OPTION', 'BtnRetirerAssociation', 'Retirer');

                    { Options Onglet 2 }
            TabbedNotebook1.Pages.Strings[1] := ReadString('OPTION', 'Page1', 'Rpertoires');

            GroupBox1.Caption        := ReadString('OPTION', 'GroupBox1', 'Rpertoire de base');
            ButtonSupprRep.Caption   := ReadString('OPTION', 'ButtonSupprRep', 'Supprimer');
            ButtonAjouterRep.Caption := ReadString('OPTION', 'ButtonAjouterRep', 'Ajouter');

                    { Options Onglet 3 }
            TabbedNotebook1.Pages.Strings[2] := ReadString('OPTION', 'Page2', 'Affichages');

            GroupBox4.Caption      := ReadString('OPTION', 'GroupBox4', 'Couleurs liste des rpertoires');
            BtnClFondRep.Caption   := ReadString('OPTION', 'BtnClFondRep', 'Fond');
            BtnClTexteRep.Caption  := ReadString('OPTION', 'BtnClTexteRep', 'Texte');
            BtnClSurvolRep.Caption := ReadString('OPTION', 'BtnClSurvolRep', 'Survol');
            LabelDemoRep.Caption   := ReadString('OPTION', 'LabelDemoRep', 'Exemple');

            GroupBox2.Caption         := ReadString('OPTION', 'GroupBox2', 'Couleurs liste de lecture');
            BtnClFondListe.Caption    := ReadString('OPTION', 'BtnClFondListe', 'Fond');
            BtnClTexteListe.Caption   := ReadString('OPTION', 'BtnClTexteListe', 'Texte');
            BtnClPlayingListe.Caption := ReadString('OPTION', 'BtnClPlayingListe', 'Charg');
            LabelDemoListe.Caption    := ReadString('OPTION', 'LabelDemoListe', 'Exemple');

            GroupBox5.Caption        := ReadString('OPTION', 'GroupBox5', 'Couleurs des barres');
            BtnColorProgress.Caption := ReadString('OPTION', 'BtnColorProgress', 'Progression');
            BtnColorVolume.Caption   := ReadString('OPTION', 'BtnColorVolume', 'Volume');

            GroupBox9.Caption       := ReadString('OPTION', 'GroupBox9', 'Icne du programme');
            BtnChangerIcone.Caption := ReadString('OPTION', 'BtnChangerIcone', 'Changer');
            BtnIconeDefaut.Caption  := ReadString('OPTION', 'BtnIconeDefaut', 'Par dfaut');

                    { Options Onglet 4 }
            TabbedNotebook1.Pages.Strings[3] := ReadString('OPTION', 'Page3', 'A propos...');
            RichEdit1.Lines.Clear;
            RichEdit1.SelAttributes.Style := [fsBold];
            RichEdit1.Lines.Add('DKiBulle v' + Version + ' par Fred Montin');
            RichEdit1.Lines.Add('');
            i := 100;
            while ValueExists('APROPOS', IntToStr(i)) do
            begin
                RichEdit1.Lines.Add(ReadString('APROPOS',  inttostr(i), ''));
                Inc(i);
            end;
            SendMessage(RichEdit1.Handle, WM_VSCROLL, SB_TOP, 0); // On scroll tout en haut
            LabelWebVersion.Caption := ReadString('OPTION', 'LabelWebVersion', 'Nouvelle version ?');
            LabelLicense.Caption    := ReadString('OPTION', 'LabelLicense', 'Afficher licence');
            end;
        end;
        if (Index = 0) or (Index = 3) then
        begin
                    { Form ID3 Tag }
            with FormTag do
            begin
            CheckArtistv1.Caption := ReadString('ID3TAG', 'CheckArtist', 'Artiste');
            CheckAlbumv1.Caption  := ReadString('ID3TAG', 'CheckAlbum', 'Album');
            CheckTitrev1.Caption  := ReadString('ID3TAG', 'CheckTitre', 'Titre');
            CheckAnneev1.Caption  := ReadString('ID3TAG', 'CheckAnnee', 'Anne');
            CheckComv1.Caption    := ReadString('ID3TAG', 'CheckCom', 'Commentaires');
            CheckGenrev1.Caption  := ReadString('ID3TAG', 'CheckGenre', 'Genre');
            CheckArtistv2.Caption := CheckArtistv1.Caption;
            CheckAlbumv2.Caption  := CheckAlbumv1.Caption;
            CheckTitrev2.Caption  := CheckTitrev1.Caption;
            CheckAnneev2.Caption  := CheckAnneev1.Caption;
            CheckComv2.Caption    := CheckComv1.Caption;
            CheckGenrev2.Caption  := CheckGenrev1.Caption;
            GroupBox3.Caption     := ReadString('ID3TAG', 'GroupBox3', 'Infos fichier');
            GroupBox4.Caption     := ReadString('ID3TAG', 'GroupBox4', 'Liste des fichiers');
            BtnSave.Caption       := ReadString('ID3TAG', 'BtnSave', 'Enregistrer');
            BtnClose.Caption      := ReadString('ID3TAG', 'BtnClose', 'Fermer');
            end;
        end;
        if (Index = 0) or (Index = 4) then
        begin
                    { Form Equalizer }
            FormEqualizer.Caption := ReadString('MAIN', 'SpdBtnEgaliseur', 'Egaliseur');
        end;
        Free;
    end;   // fin with IniLangue
end;

//============================================================================//
{                  Mise  jours des couleurs et des icnes                     }
//----------------------------------------------------------------------------//
procedure MiseaJoursCouleurs(Totale : Boolean = True);
var
    Ini: Tinifile;
begin
            { Mise  jours des couleurs }
    Ini := Tinifile.Create(FichierIni);
    with Ini do
    begin
                { Form 1 }
      With Form1 do
      begin
        TreeView1.Color          := ReadInteger('DATA', 'BtnClFondRep', 0);
        TreeView1.Font.Color     := ReadInteger('DATA', 'BtnClTexteRep', 0);
        TreeView1.Tag            := ReadInteger('DATA', 'BtnClSurvolRep', 0);
        Listbox1.Color           := ReadInteger('DATA', 'BtnClFondListe', 0);
        Listbox1.Font.Color      := ReadInteger('DATA', 'BtnClTexteListe', 0);
        Listbox1.Tag             := ReadInteger('DATA', 'BtnClPlayingListe', 0);
        ProgressBar1.Brush.Color := clGray;
        ProgressBar2.Brush.Color := clGray;
        SendMessage(ProgressBar1.Handle, WM_USER + 9, 0, ReadInteger('DATA', 'BtnColorProgress', 0));
        SendMessage(ProgressBar2.Handle, WM_USER + 9, 0, ReadInteger('DATA', 'BtnColorVolume', 0));
        ListBox1.Repaint;
        TreeView1.Repaint;
                { Form 2 }
        if Totale then
        begin
          With Form2 do
          begin
            LabelDemoRep.Color        := TreeView1.Color;
            LabelDemoRep.Font.Color   := TreeView1.Font.Color;
            LabelDemoListe.Color      := Listbox1.Color;
            LabelDemoListe.Font.Color := Listbox1.Font.Color;
          end;
        end;
      end;
                { Icones }
      With Form1.TrayIcon1 do
      begin
          if FileExists(ReadString('DATA', 'Icone', '')) then
              Icon.LoadFromFile(ReadString('DATA', 'Icone', ''))
          else
              Icon := Form1.Icon;
          BackgroundIcon := Icon;
      end;
    end;
    Ini.Free;
end;

//============================================================================//
{                  Executer une commande                                       }
//----------------------------------------------------------------------------//
procedure Executer(const action : string);
begin
    Screen.Cursor := crHourGlass;
    ShellExecute(Form1.Handle, Nil, PChar(action), Nil, Nil, SW_SHOWNORMAL);
    Screen.Cursor := crDefault;
end;

//============================================================================//
{                  Renvoi le genre d'un chanson (ID3v1)                        }
//----------------------------------------------------------------------------//
function GetGenreFromIndex(Index : integer) : string;
begin
    if ((Index < 0) or (Index > 80)) then Index := 80;
    Result := SGenre[Index];
end;

function GetIndexofGenre(const Genre : String) : byte;
var
    i : integer;
begin
    for i := Low(SGenre) to High(SGenre) do
    begin
        if LowerCase(Genre) = LowerCase(SGenre[i]) then
        begin
            Result := i;
            Exit;
        end;
    end;
    Result := 80;
end;


//============================================================================//
{                  Active l'galiseur                                          }
//----------------------------------------------------------------------------//
procedure SetEqualizer;
begin
  With Form1 do
    if Channel <> 0 then
    begin
        fx[1] := BASS_ChannelSetFX(Form1.Channel, BASS_FX_PARAMEQ, 1);
        fx[2] := BASS_ChannelSetFX(Form1.Channel, BASS_FX_PARAMEQ, 1);
        fx[3] := BASS_ChannelSetFX(Form1.Channel, BASS_FX_PARAMEQ, 1);
        p.fBandwidth := 18;
        p.fGain := eq[1];
        p.fCenter := 125;
        BASS_FXSetParameters(fx[1], @p);
        p.fGain := eq[2];
        p.fCenter := 1000;
        BASS_FXSetParameters(fx[2], @p);
        p.fGain := eq[3];
        p.fCenter := 8000;
        BASS_FXSetParameters(fx[3], @p);
    end;
end;

end.
